from holger.elasticsearch.client import ElasticSearchClient
import uuid
import datetime
from sentry_sdk import capture_event
from holger.sentry.utils.functions import config_scope, Level


def log(response, status_code, index: str, doc, params=None, headers=None):
    if headers is None:
        headers = {}
    if params is None:
        params = {}
    status = response.get('status')
    metadata = response.get('metadata')
    transaction = metadata.pop('transaction', {})
    created_at = datetime.datetime.now()
    client = ElasticSearchClient.get_client()
    elastic_id = metadata.get('id', F"{uuid.uuid4()}-{created_at.timestamp()}")
    client.index(
        index=index,
        body={
            **response,
            'transaction': transaction,
            'created_at': created_at
        },
        doc_type=doc,
        id=elastic_id,
        params=params,
        headers=headers
    )
    if status == 'failed':
        config_scope(metadata, status_code, Level.ERROR)
        sentry_exception_code = capture_event({
            **response.get('error')
        })
    elif status == 'success':
        config_scope(metadata, status_code, Level.INFO)
        sentry_exception_code = capture_event(
            {
                **response.get('data'),
                'message': 'request was successfully submitted'
            }
        )
    else:
        config_scope(metadata, status_code, Level.ERROR)
        sentry_exception_code = capture_event(
            {
                **response,
                'message': F'status is invalid ({status})'
            }
        )
    metadata.update({
        'transaction': {
            'status': status,
            'index': index,
            'elastic_id': elastic_id,
            'sentry_code': sentry_exception_code
        }
    })



from rest_framework import pagination
from django.http import JsonResponse
import json


class Pagination(pagination.PageNumberPagination):
    def get_paginated_response(self, data):
        return JsonResponse({
            "count": self.page.paginator.count,
            'current': self.page.number,
            "data": json.loads(json.dumps(data))
        })



from functools import wraps
from ..functions import log as log_provider


def log(index: str, doc, params=None, headers=None):
    def decorator(func):
        @wraps(func)
        def inner(*args, **kwargs):
            response, status_code = func(*args, **kwargs)
            log_provider(response, status_code, index, doc, params, headers)
            return response, status_code

        return inner

    return decorator



from django.conf import settings

PROTOCOL = getattr(settings, 'ELASTIC_PROTOCOL', 'http')
HOST = getattr(settings, 'ELASTIC_HOST', 'localhost')
PORT = getattr(settings, 'ELASTIC_PORT', 9200)
USE_SSL = getattr(settings, 'ELASTIC_USE_SSL', False)



import setuptools

with open("holger/README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="holger-utils",
    version="0.1.36",
    author="Mohammad Hosein Shamsaei",
    author_email="holgerco.dev@gmail.com",
    description="utils for developing web application",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/football-fantasy/holger-utils",
    install_requires=[
        'requests',
        'djangorestframework',
        'elasticsearch[async]',
        'celery',
        'sentry-sdk'
    ],
    packages=setuptools.find_packages(),
    classifiers=[
        'Environment :: Web Environment',
        "Programming Language :: Python :: 3",
        'License :: OSI Approved :: MIT License',
        "Operating System :: OS Independent",
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
    ],
    python_requires='>=3.6',
)

# py setup.py sdist bdist_wheel
# twine upload dist/*
